/*
 * The contents of this file are subject to the AOLserver Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://aolserver.lcs.mit.edu/.
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is AOLserver Code and related documentation
 * distributed by AOL.
 * 
 * The Initial Developer of the Original Code is America Online,
 * Inc. Portions created by AOL are Copyright (C) 1999 America Online,
 * Inc. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms
 * of the GNU General Public License (the "GPL"), in which case the
 * provisions of GPL are applicable instead of those above.  If you wish
 * to allow use of your version of this file only under the terms of the
 * GPL and not to allow others to use your version of this file under the
 * License, indicate your decision by deleting the provisions above and
 * replace them with the notice and other provisions required by the GPL.
 * If you do not delete the provisions above, a recipient may use your
 * version of this file under either the License or the GPL.
 */


/*
 * conn7.c --
 *
 *      Tcl 7 version of some Conn routines
 */

static const char *RCSID = "@(#) $Header: /usr/local/cvsroot/nsd/aolserver/nsd/conn7.c,v 1.2 2000/06/29 20:15:14 mayoff Exp $, compiled: " __DATE__ " " __TIME__;

#include "nsd.h"


/*
 *----------------------------------------------------------------------
 *
 * Ns_WriteConn --
 *
 *	This will write a buffer to the conn. It promises to write 
 *	all of it. 
 *
 * Results:
 *	NS_OK/NS_ERROR
 *
 * Side effects:
 *	Stuff may be written 
 *
 *----------------------------------------------------------------------
 */

int
Ns_WriteConn(Ns_Conn *conn, char *buf, int len)
{
    return NsWriteConnRaw(conn, buf, len);
}


/* 
 *==========================================================================
 * Exported functions
 *==========================================================================
 */

/*
 *----------------------------------------------------------------------
 *
 * NsCharsetInit --
 *
 *      Initialize charset translation support.  NOTE: charsets are
 *      not supported in nsd76.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

void
NsCharsetInit(void)
{
}


/*
 *----------------------------------------------------------------------
 *
 * NsConnComputeEncoding --
 *
 *      Figure out what charset the client will expect, based on the
 *      content type.  NOTE: charsets are not supported in nsd76.
 *
 * Results:
 *	NS_OK
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

void
NsConnComputeEncoding(Ns_Conn *conn)
{
}


/*
 *----------------------------------------------------------------------
 *
 * NsConnEncodeForOutput --
 *
 *      Translate the buffer to the output encoding for the connection.
 *      NOT IMPLEMENTED for nsd76.
 *
 * Results:
 *	Fatal error - does not return.
 *
 * Side effects:
 *	Aborts the program.
 *
 *----------------------------------------------------------------------
 */

void
NsConnEncodeForOutput(Ns_Conn *conn, char *buf, int len, Tcl_DString *ds)
{
    Ns_Fatal("NsConnEncodeForOutput should never be called in nsd76!");
}

