#!/bin/bash -e

globMatch ()
{
    case "$1" in
	$2) return 0 ;;
	*) return 1 ;;
    esac
}

while globMatch "$1" "-*"; do
    case "$1" in
	-\?)
	    echo 1>&2 "Usage: make-src-dist.sh [-k]"
	    exit 1
	    ;;

	-k)
	    keep=1
	    ;;

	*)
	    echo 1>&2 "Unknown flag: $1"
	    exit 1
	    ;;
    esac
    shift
done

if [ $# != "0" ]; then
    echo "Unknown argument: $1"
    exit 1
fi

cat <<EOF
AOLserver Release Checklist:

- Update version number and patch descriptions in
  ../aolserver/DISTRIBUTION.txt

- Update NSD_VERSION and NSD_LABEL in ../aolserver/nsd/nsd.h

- Commit those changes & tag new revisions stable

- Tag stable revisions as new release, e.g.
  cd ../aolserver && cvs tag -r stable release-3_2_ad10

- Update ./CONTENTS, commit it, tag it stable, tag it with a new release
  tag.  Note that CONTENTS should list its own new release tag (on
  the arsdigita-aol3 line) *before* it has been tagged.

EOF

echo "Building source distribution using these module tags:"
cat CONTENTS
echo ""

if [ "$keep" = "" ]; then
    rm -rf root
fi
[ -d root ] || mkdir root

cat CONTENTS | while read module tag; do
    (
	echo "Checking out $module..."
	cd root && cvs -Q checkout -P -r $tag -d $module nsd/$module
    )
done

echo "Generating aolserver patch..."
as_tag="$(awk '$1 == "aolserver" {print $2}' < CONTENTS)"

(
    cd root && cvs -Q diff -r aolserver -r $as_tag aolserver
) > root/aolserver/DISTRIBUTION.patch

echo "Creating archive..."
tar -c -f - --exclude=CVS root | gzip -9 > aolserver-src.tar.gz

echo "Done."
exit 0
